@@ -1,34 +1,90 @@
 """Attribute selector plugin.
 
-Simple syntax (-a, --attr) examples:
+Oftentimes when testing you will want to select tests based on
+criteria rather then simply by filename. For example, you might want
+to run all tests except for the slow ones. You can do this with the
+Attribute selector plugin by setting attributes on your test methods.
+Here is an example:
 
-* `nosetests -a status=stable`:
-   Only test cases with attribute "status" having value "stable"
+.. code-block:: python
 
-* `nosetests -a priority=2,status=stable`:
-   Both attributes must match
+    def test_big_download():
+        import urllib
+        # commence slowness...
 
-* `nosetests -a priority=2 -a slow`:
-   Either attribute must match
-    
-* `nosetests -a tags=http`:
-   Attribute list "tags" must contain value "http" (see test_foobar()
-   below for definition)
+    test_big_download.slow = 1
 
-* `nosetests -a slow`:
-   Attribute "slow" must be defined and its value cannot equal to False
+Once you've assigned an attribute ``slow = 1`` you can exclude that
+test and all other tests having the slow attribute by running ::
+
+    $ nosetests -a '!slow'
+
+There is also a decorator available for you that will set attributes.
+Here's how to set ``slow=1`` like above with the decorator:
+
+.. code-block:: python
+
+    from nose.plugins.attrib import attr
+    @attr('slow')
+    def test_big_download():
+        import urllib
+        # commence slowness...
+
+And here's how to set an attribute with a specific value:
+
+.. code-block:: python
+
+    from nose.plugins.attrib import attr
+    @attr(speed='slow')
+    def test_big_download():
+        import urllib
+        # commence slowness...
+
+This test could be run with ::
+
+    $ nosetests -a speed=slow
+
+Below is a reference to the different syntaxes available.
+
+Simple syntax
+-------------
+
+Examples of using the ``-a`` and ``--attr`` options:
+
+* ``nosetests -a status=stable``
+   Only runs tests with attribute "status" having value "stable"
+
+* ``nosetests -a priority=2,status=stable``
+   Runs tests having both attributes and values
+
+* ``nosetests -a priority=2 -a slow``
+   Runs tests that match either attribute
+
+* ``nosetests -a tags=http``
+   If a test's ``tags`` attribute was a list and it contained the value
+   ``http`` then it would be run
+
+* ``nosetests -a slow``
+   Runs tests with the attribute ``slow`` if its value does not equal False
    (False, [], "", etc...)
 
-* `nosetests -a '!slow'`:
-   Attribute "slow" must NOT be defined or its value must be equal to False.  
-   Note the quotes around the value -- this may be necessary if your shell 
-   interprets `!' as a special character.
+* ``nosetests -a '!slow'``
+   Runs tests that do NOT have the attribute ``slow`` or have a ``slow``
+   attribute that is equal to False
+   **NOTE**:
+   if your shell (like bash) interprets '!' as a special character make sure to
+   put single quotes around it.
 
-Eval expression syntax (-A, --eval-attr) examples:
+Expression Evaluation
+---------------------
 
-* `nosetests -A "not slow"`
+Examples using the ``-A`` and ``--eval-attr`` options:
 
-* `nosetests -A "(priority > 5) and not slow"`
+* ``nosetests -A "not slow"``
+  Evaluates the Python expression "not slow" and runs the test if True
+
+* ``nosetests -A "(priority > 5) and not slow"``
+  Evaluates a complex Python expression and runs the test if True
 
 """
 import logging
@@ -41,18 +97,30 @@ from nose.util import tolist
 log = logging.getLogger('nose.plugins.attrib')
 compat_24 = sys.version_info >= (2, 4)
 
+def attr(*args, **kwargs):
+    """Decorator that adds attributes to objects
+    for use with the Attribute (-a) plugin.
+    """
+    def wrap(func):
+        for name in args:
+            # these are just True flags:
+            setattr(func, name, 1)
+        func.__dict__.update(kwargs)
+        return func
+    return wrap
+
 class ContextHelper:
     """Returns default values for dictionary lookups."""
     def __init__(self, obj):
         self.obj = obj
-        
+
     def __getitem__(self, name):
         return self.obj.get(name, False)
 
 
 class AttributeGetter:
     """Helper for looking up attributes
-    
+
     First we check the method, and if the attribute is not present,
     we check the method's class.
     """
@@ -79,13 +147,13 @@ class AttributeSelector(Plugin):
     def __init__(self):
         Plugin.__init__(self)
         self.attribs = []
-    
-    def options(self, parser, env=os.environ):
-        """Add command-line options for this plugin."""
 
+    def options(self, parser, env):
+        """Register command line options"""
         parser.add_option("-a", "--attr",
                           dest="attr", action="append",
                           default=env.get('NOSE_ATTR'),
+                          metavar="ATTR",
                           help="Run only tests that have attributes "
                           "specified by ATTR [NOSE_ATTR]")
         # disable in < 2.4: eval can't take needed args
@@ -107,7 +175,7 @@ class AttributeSelector(Plugin):
         match.
         """
         self.attribs = []
-        
+
         # handle python eval-expression parameter
         if compat_24 and options.eval_attr:
             eval_attr = tolist(options.eval_attr)
@@ -149,7 +217,7 @@ class AttributeSelector(Plugin):
                 self.attribs.append(attr_group)
         if self.attribs:
             self.enabled = True
-            
+
     def validateAttrib(self, attribs):
         # TODO: is there a need for case-sensitive value comparison?
         # within each group, all must match for the group to match
@@ -176,7 +244,9 @@ class AttributeSelector(Plugin):
                         break
                 elif type(obj_value) in (list, tuple):
                     # value must be found in the list attribute
-                    if not value in [str(x).lower() for x in obj_value]:
+
+                    if not str(value).lower() in [str(x).lower()
+                                                  for x in obj_value]:
                         match = False
                         break
                 else:
@@ -205,10 +275,14 @@ class AttributeSelector(Plugin):
         if wanted:
             return None
         return False
-        
+
     def wantFunction(self, function):
+        """Accept the function if its attributes match.
+        """
         return self.validateAttrib(function.__dict__)
-        
+
     def wantMethod(self, method):
+        """Accept the method if its attributes match.
+        """
         attribs = AttributeGetter(method.im_class, method)
         return self.validateAttrib(attribs)
