@@ -8,15 +8,30 @@ import re
 import sys
 import types
 import unittest
-from compiler.consts import CO_GENERATOR
 from types import ClassType, TypeType
 
+try:
+    from compiler.consts import CO_GENERATOR
+except ImportError:
+    # IronPython doesn't have a complier module
+    CO_GENERATOR=0x20
+    
 log = logging.getLogger('nose')
 
 ident_re = re.compile(r'^[A-Za-z_][A-Za-z0-9_.]*$')
 class_types = (ClassType, TypeType)
 skip_pattern = r"(?:\.svn)|(?:[^.]+\.py[co])|(?:.*~)|(?:.*\$py\.class)"
 
+try:
+    set()
+    set = set # make from nose.util import set happy
+except NameError:
+    try:
+        from sets import Set as set
+    except ImportError:
+        pass
+
+
 def ls_tree(dir_path="",
             skip_pattern=skip_pattern,
             indent="|-- ", branch_indent="|   ",
@@ -134,7 +149,7 @@ def file_like(name):
 
 def cmp_lineno(a, b):
     """Compare functions by their line numbers.
-    
+
     >>> cmp_lineno(isgenerator, ispackage)
     -1
     >>> cmp_lineno(ispackage, isgenerator)
@@ -159,7 +174,7 @@ def func_lineno(func):
 
 
 def isclass(obj):
-    """Is obj a class? inspect's isclass is too liberal and returns True
+    """Is obj a class? Inspect's isclass is too liberal and returns True
     for objects that can't be subclasses of anything.
     """
     obj_type = type(obj)
@@ -202,6 +217,25 @@ def ispackage(path):
     return False
 
 
+def isproperty(obj):
+    """
+    Is this a property?
+
+    >>> class Foo:
+    ...     def got(self):
+    ...         return 2
+    ...     def get(self):
+    ...         return 1
+    ...     get = property(get)
+
+    >>> isproperty(Foo.got)
+    False
+    >>> isproperty(Foo.get)
+    True
+    """
+    return type(obj) == property
+
+
 def getfilename(package, relativeTo=None):
     """Find the python source file for a package, relative to a
     particular directory (defaults to current working directory if not
@@ -216,13 +250,13 @@ def getfilename(package, relativeTo=None):
         if os.path.exists(filename):
             return filename
     return None
-    
+
 
 def getpackage(filename):
     """
     Find the full dotted package name for a given python source file
     name. Returns None if the file is not a python source file.
-    
+
     >>> getpackage('foo.py')
     'foo'
     >>> getpackage('biff/baf.py')
@@ -373,7 +407,7 @@ def split_test_name(test):
         return (None, None, fn)
 split_test_name.__test__ = False # do not collect
 
-    
+
 def test_address(test):
     """Find the test address for a test, which may be a module, filename,
     class, method or function.
@@ -387,7 +421,7 @@ def test_address(test):
     if t == types.ModuleType:
         file = getattr(test, '__file__', None)
         module = getattr(test, '__name__', None)
-        return (file, module, call)
+        return (src(file), module, call)
     if t == types.FunctionType or issubclass(t, type) or t == types.ClassType:
         module = getattr(test, '__module__', None)
         if module is not None:
@@ -396,18 +430,22 @@ def test_address(test):
             if file is not None:
                 file = os.path.abspath(file)
         call = getattr(test, '__name__', None)
-        return (file, module, call)
+        return (src(file), module, call)
     if t == types.InstanceType:
         return test_address(test.__class__)
     if t == types.MethodType:
         cls_adr = test_address(test.im_class)
-        return (cls_adr[0], cls_adr[1],
+        return (src(cls_adr[0]), cls_adr[1],
                 "%s.%s" % (cls_adr[2], test.__name__))
     # handle unittest.TestCase instances
     if isinstance(test, unittest.TestCase):
-        if hasattr(test, '_FunctionTestCase__testFunc'):
+        if (hasattr(test, '_FunctionTestCase__testFunc') # pre 2.7
+            or hasattr(test, '_testFunc')):              # 2.7
             # unittest FunctionTestCase
-            return test_address(test._FunctionTestCase__testFunc)
+            try:
+                return test_address(test._FunctionTestCase__testFunc)
+            except AttributeError:
+                return test_address(test._testFunc)
         # regular unittest.TestCase
         cls_adr = test_address(test.__class__)
         # 2.5 compat: __testMethodName changed to _testMethodName
@@ -415,7 +453,7 @@ def test_address(test):
             method_name = test._TestCase__testMethodName
         except AttributeError:
             method_name = test._testMethodName
-        return (cls_adr[0], cls_adr[1],
+        return (src(cls_adr[0]), cls_adr[1],
                 "%s.%s" % (cls_adr[2], method_name))
     raise TypeError("I don't know what %s is (%s)" % (test, t))
 test_address.__test__ = False # do not collect
@@ -445,9 +483,9 @@ def try_run(obj, names):
                         raise TypeError("Attribute %s of %r is not a python "
                                         "function. Only functions or callables"
                                         " may be used as fixtures." %
-                                        (name, obj))                    
+                                        (name, obj))
                 if len(args):
-                    log.debug("call fixture %s.%s(%s)", obj, name, obj)    
+                    log.debug("call fixture %s.%s(%s)", obj, name, obj)
                     return func(obj)
             log.debug("call fixture %s.%s", obj, name)
             return func()
@@ -486,7 +524,7 @@ def match_last(a, b, regex):
         return -1
     return cmp(a, b)
 
-        
+
 def tolist(val):
     """Convert a value that may be a list or a (possibly comma-separated)
     string into a list. The exception: None is returned as None, not [None].
@@ -510,7 +548,7 @@ def tolist(val):
     try:
         return re.split(r'\s*,\s*', val)
     except TypeError:
-        # who knows... 
+        # who knows...
         return list(val)
 
 
@@ -578,7 +616,7 @@ def transplant_func(func, module):
     >>> pp.__module__
     'nose.util'
 
-    The original function is not modified
+    The original function is not modified.
 
     >>> pprint.__module__
     'pprint'
@@ -594,7 +632,7 @@ def transplant_func(func, module):
     from nose.tools import make_decorator
     def newfunc(*arg, **kw):
         return func(*arg, **kw)
-    
+
     newfunc = make_decorator(func)(newfunc)
     newfunc.__module__ = module
     return newfunc
@@ -613,7 +651,7 @@ def transplant_class(cls, module):
     'nose.util'
     >>> Nf.__name__
     'Failure'
-    
+
     """
     class C(cls):
         pass
@@ -622,6 +660,16 @@ def transplant_class(cls, module):
     return C
 
 
+def safe_str(val, encoding='utf-8'):
+    try:
+        return str(val)
+    except UnicodeEncodeError:
+        if isinstance(val, Exception):
+            return ' '.join([safe_str(arg, encoding)
+                             for arg in val])
+        return unicode(val).encode(encoding)
+
+    
 if __name__ == '__main__':
     import doctest
     doctest.testmod()
